/*
 *
 * This source file belongs to the Sensoria CASE tool developed at the Institute of 
 * Computer Science, Programming and Software Engineering, of the Ludwig-Maximilians-
 * Universität München, Germany. 
 *
 * See the enclosed LICENSE file for licensing information.
 *
 */
package com.uppaal.verifyta.ws;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * This class implements the UPPAAL Verify TA Web Service.
 * 
 * @author Max Zeumer
 * 
 */
public class RunVerifyTA {

	public static final int PROPERTYCHECK= 1;

	public static final int TRACE= 2;

	public static final int LOG= 0;

	private final String uppaalLocation= "verifyta";

	private final String tmpFolderLocation= "tmp/uppaal/";

	private final boolean deleteFiles= false;

	public String[] verifyTA(String tafilecontent, String qfilecontent, String options) {

		// set up Strings to return later
		String[] result= { "", "", "" };

		// make sure both parameters provide information
		if (tafilecontent == null || tafilecontent.equals("")) {
			result[PROPERTYCHECK]= "[ERROR: verifyTA] A system string must be provided. ";
			return result;
		}
		;
		if (qfilecontent == null || qfilecontent.equals("")) {
			result[PROPERTYCHECK]= "[ERROR: verifyTA] A query string must be provided. ";
			return result;
		}
		;

		// create a file handler for this thread
		FileHandler fileHandler= new DebugFileHandler(tmpFolderLocation, deleteFiles);

		// create the temp files
		File taFile= null;
		File qFile= null;
		try {
			taFile= fileHandler.createFile("uppaal", "ta");
			qFile= fileHandler.createFile("uppaal", "q");
			// write the parameters to the temp files
			fileHandler.writeToFile(tafilecontent, taFile);
			fileHandler.writeToFile(qfilecontent, qFile);
		} catch (IOException e) {
			// instantly return error message
			result[PROPERTYCHECK]= "[ERROR: verifyTA] An Exception occured while processing your request. See log for details. \n";
			try {
				fileHandler.deleteFiles();
			} catch (IOException e1) {
				result[LOG]+= "[ERROR: verifyTA] " + e1.getMessage() + "\n";
			}
			result[LOG]+= "[ERROR: verifyTA] " + e.getMessage();
			for (int i= 0; i < e.getStackTrace().length; i++) {
				result[LOG]+= "[JAVA: verifyTA] " + e.getStackTrace()[i].toString() + "\n";
			}
			return result;
		}

		// run uppaal
		try {
			String call;
			if (options != null && !options.equals("")) {
				call= uppaalLocation + " " + options + " " + taFile.getAbsolutePath() + " " + qFile.getAbsolutePath();
			} else {
				call= uppaalLocation + " -t 1 -s -q " + taFile.getAbsolutePath() + " " + qFile.getAbsolutePath();
			}
			result[LOG]+= "[LOG: verifyTA] Verifying the model (" + call + ")\n";
			java.lang.Process process= Runtime.getRuntime().exec(call);
			result[TRACE]= getTrail(process);

			// try to guess wheather the result[TRACE] contains an error message
			// or a trace
			if (!result[TRACE].contains("State") || result[TRACE].contains("Bad property") || result[TRACE].contains("Unknown identifier")
					|| result[TRACE].contains("syntax error")) {
				result[LOG]+= "[LOG: verifyTA] " + result[TRACE] + "\n";
				result[TRACE]= "";
			}

			String output= getProcessOutput(process).trim();
			result[LOG]+= output + "\n";
			output= filterOutput(output).trim();

			process.destroy();

			// analyze the result
			if (output.contains("Property") && output.contains("line")) {
				result[PROPERTYCHECK]= output;
			} else {
				if (output.contains("Property") && output.contains("NOT")) {
					result[PROPERTYCHECK]= "[CHECK: verifyTA] Property is NOT satisfied \n";
				} else if (output.contains("Property") && output.contains("MAYBE")) {
					result[PROPERTYCHECK]= "[CHECK: verifyTA] Property is MAYBE satisfied \n";
				} else {
					result[PROPERTYCHECK]= "[CHECK: verifyTA] Property is satisfied";
				}
			}

		} catch (Exception e) {
			// instantly return error message
			result[PROPERTYCHECK]= "[ERROR: verifyTA] An Exception occured while processing your request. See log for details. ";
			try {
				fileHandler.deleteFiles();
			} catch (IOException e1) {
				result[LOG]+= "[ERROR: verifyTA]" + e1.getMessage();
			}
			result[LOG]+= "[ERROR: verifyTA]" + e.getMessage();
			for (int i= 0; i < e.getStackTrace().length; i++) {
				result[LOG]+= "[JAVA: verifyTA] " + e.getStackTrace()[i].toString() + "\n";
			}
			return result;
		}



		return result;
	}

	private String filterOutput(String output) throws IOException {
		String filteredOutput= "";
		String line;

		BufferedReader reader= new BufferedReader(new InputStreamReader(new ByteArrayInputStream(output.getBytes())));
		while ( (line= reader.readLine()) != null) {
			if (line.contains("Property") && line.contains("line"))
				filteredOutput+= line + "\n";
		}

		return filteredOutput;
	}

	private String getTrail(java.lang.Process process) throws IOException {
		String trail= "";
		String line;
		BufferedReader errorReader= new BufferedReader(new InputStreamReader(process.getErrorStream()));
		while ( (line= errorReader.readLine()) != null) {
			trail+= line + "\n";
		}
		return trail;
	}

	private String getProcessOutput(java.lang.Process process) throws IOException {
		String output= "";
		String line;
		BufferedReader outputReader= new BufferedReader(new InputStreamReader(process.getInputStream()));
		while ( (line= outputReader.readLine()) != null) {
			output+= "[UPPAAL: verifyTA] " + line + "\n";
		}
		return output;
	}
}
